/* SCE CONFIDENTIAL
 * PlayStation(R)Suite SDK 0.98.2
 * Copyright (C) 2012 Sony Computer Entertainment Inc.
 * All Rights Reserved.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using Sce.Pss.Core;
using Sce.Pss.Core.Environment;
using Sce.Pss.Core.Graphics;
using Sce.Pss.Core.Imaging;
using Sce.Pss.Core.Input;

namespace AoBench
{
	public class AppMain
	{
		static GamePadData pad;
		static Vector4 Param;
		static protected GraphicsContext graphics;
		static VertexBuffer vertexBuffer;
		static ShaderProgram shaderProgram;
		static ShaderProgram shaderPost;
    	static FrameBuffer frameBuffer;
    	static Texture2D renderTexture;
    	static DepthBuffer depthBuffer;
		static Stopwatch stopwatch = new Stopwatch();
		
	    const int offscreenWidth = 256;
	    const int offscreenHeight = 256;
	    
		/*
		const int offscreenWidth = 384;
	    const int offscreenHeight = 192;
	    */
		/*
		const int offscreenWidth = 960;
	    const int offscreenHeight = 544;
	    */
		
		static float[] vertices =
		{
			-1.0f,  1.0f, 0.0f,
			-1.0f, -1.0f, 0.0f,
			 1.0f, -1.0f, 0.0f,
			 1.0f,  1.0f, 0.0f,
		};
		
		static bool once_ao = false;
		
		public static void Term()
		{
			vertexBuffer.Dispose();
			shaderProgram.Dispose();
			shaderPost.Dispose();
			frameBuffer.Dispose();
			graphics.Dispose();
		}
		
		public static void Main (string[] args)
		{
			Initialize ();
			pad = GamePad.GetData(0);
			bool bQuit = false;

			while (!bQuit)
			{
				SystemEvents.CheckEvents ();
				Update ();
				Render ();
				pad = GamePad.GetData(0);
				if( (pad.Buttons & GamePadButtons.Cross) != 0)
				{
					bQuit = true;
				}
			}
			Term();
		}

		public static void Initialize ()
		{
			graphics		= new GraphicsContext();
            vertexBuffer	= new VertexBuffer(4, VertexFormat.Float3);
            vertexBuffer.SetVertices(0, vertices);
  
			shaderProgram	= new ShaderProgram("/Application/shader/Tech.cgx");
			shaderPost		= new ShaderProgram("/Application/shader/Post.cgx");
			shaderProgram.SetUniformBinding(0, "Time");

			frameBuffer		= new FrameBuffer();
        	renderTexture	= new Texture2D(offscreenWidth, offscreenHeight, false, PixelFormat.Rgba, PixelBufferOption.Renderable);
        	depthBuffer		= new DepthBuffer(offscreenWidth, offscreenHeight, PixelFormat.Depth16);
        	frameBuffer.SetColorTarget(renderTexture, 0);
        	frameBuffer.SetDepthTarget(depthBuffer);
		}

		public static void Update ()
		{
			shaderProgram.SetUniformValue(0, ref Param);
		}
		
		public static void DrawRect ()
		{
			FrameBuffer currentBuffer = graphics.GetFrameBuffer();
        	graphics.SetViewport(0, 0, currentBuffer.Width, currentBuffer.Height);
        	graphics.SetClearColor(0.0f, 0.5f, 1.0f, 1.0f);
        	graphics.Clear();
			graphics.SetVertexBuffer(0, vertexBuffer);
			graphics.DrawArrays(DrawMode.TriangleFan, 0, 4);
		}
		
		public static void Render ()
		{
			//Tech
			if(once_ao == false)
			{
				graphics.SetFrameBuffer(frameBuffer);
				graphics.SetShaderProgram(shaderProgram);
				DrawRect();
				once_ao = true;
			}
			
			//Post
			graphics.SetShaderProgram(shaderPost);
			graphics.SetFrameBuffer(graphics.Screen);
			graphics.SetTexture(0, renderTexture);
			DrawRect();

			graphics.SwapBuffers();	
		}
	}
}
